#include "Search.h"

double angle (int x1, int x2, int y1, int y2) {
  return atan2((x2-x1), (y2-y1));
}

Point radialSearch(const Point &origin, const Point &startPt, const Point &endPt, Sketch<bool> sketch) {
  int ox = origin.coordX();
  int oy = origin.coordY();
  double astart = angle(ox, startPt.coordX(), oy, startPt.coordY());
  double aend = angle(ox, endPt.coordX(), oy, endPt.coordY());

  int i,j;
  double first = DBL_MAX;
  
  Point r = origin;

  for (i = 0; i < sketch.height; i++) {
    for (j = 0; j < sketch.width; j++) {
      if (!sketch->at(i,j)) continue;

      double a = angle(ox, j, oy, i);
      if (a >= astart && a <= aend && a < first) {
	first = a;
	r = Point(i,j);
      }
    }
  }

  return r;
}

ShapeRoot radialSearch(ShapeRoot &origin, const Point &startPt, const Point &endPt, const UnaryShapeRootPred &pred) {
  int ox = origin->getCentroid().coordX();
  int oy = origin->getCentroid().coordY();
  double astart = angle(ox, startPt.coordX(), oy, startPt.coordY());
  double aend = angle(ox, endPt.coordX(), oy, endPt.coordY());

  double first = DBL_MAX;
  ShapeRoot r = origin;
  
  NEW_SHAPEROOTVEC(shapes, origin);
  SHAPEROOTVEC_ITERATE(shapes,shape)
    /*
    if ( !(pred(shape))) {
      continue;
    }
    */
    if (shape == origin) continue;
    double a = angle(ox, shape->getCentroid().coordX(), oy, shape->getCentroid().coordY());
    if (a >= astart && a <= aend && a < first) {
      first = a;
      r = shape;
    }
  END_ITERATE
  return r;
}

void set_pixel(Sketch<uint> &sketch, int i, int j, uint val) {
  sketch->getRawPixels()[sketch.height * i + j] = val;
}

std::vector<Point> contourSearch(const Point &startPt, Sketch<uint> &sketch, uint contour, uint target) {
  std::vector<Point> r;

  if (startPt.coordX() < 0 || startPt.coordX() >= sketch.width ||
      startPt.coordY() < 0 || startPt.coordY() >= sketch.height) {
    return r;
  }

  uint dummy = 0;
  while (dummy == contour || dummy == target) {
    dummy++;
  }

  int i = startPt.coordY();
  int j = startPt.coordX();

  if (sketch->at(i,j) == target) {
    r.push_back(Point(i,j));
  }
  if (sketch->at(i,j) == target || sketch->at(i,j) == contour) {
    set_pixel(sketch, i, j, dummy);
    std::vector<Point> e = contourSearch(Point(i-1,j), sketch, contour, target);
    r.insert(r.end(), e.begin(), e.end());
    e = contourSearch(Point(i+1,j), sketch, contour, target);
    r.insert(r.end(), e.begin(), e.end());
    e = contourSearch(Point(i,j-1), sketch, contour, target);
    r.insert(r.end(), e.begin(), e.end());
    e = contourSearch(Point(i,j+1), sketch, contour, target);
    r.insert(r.end(), e.begin(), e.end());
  }

  return r;
}

std::vector<ShapeRoot> contourSearch(ShapeRoot &startPt, const UnaryShapeRootPred &pred) {
  std::vector<ShapeRoot> r;
  NEW_SHAPEROOTVEC(shapes, startPt);
  SHAPEROOTVEC_ITERATE(shapes,shape)
    /*
    if (!pred(shape)) {
      continue;
    }
    */
  if (startPt->isInside(shape->getCentroid())) {
    r.push_back(shape);
  }
  END_ITERATE
  return r;
}
